%Clustering algorithm by Longin Jan Latecki dt: 10/02/02
%Written by Venugopal Rajagopal
%data clustering algorithm
%
%Modified 11/4/2002 by Jeremy W. Sheaffer to handle already loaded
%and normalized data.
%firstauto1(inputFilename,mnp,percentage)
%input:
%inputFilename: Text file, each row contains a data vector of the data to be clustered
%mnp          : Minimal number of points in each cluster
%percentage   : Parameter that delays clustering of points in a margin region (normal value .05 i.e. 5 percent)
%output:
%cluster      : A list of cluster indices assigned to data vectors

%example:
%
%[cluster] = firstauto1('irises1.dat',30,0.07);
%Modified on 11/27/02

function [cluster] = firstauto1(data,mnp,percentage) %Call function by giving input file name
                                              %minimum number of points and percentage

x=data;
d=data;
%[m, s, d] = normalize(x, [], []);    %normalize the dataset
%[datas, proj] = pca(d, [], 3);     %Extracting the principal components from the dataset

x = d;
avg = mean(x);                  %Averages of each column in the dataset x 

onemat = ones(size(x,1));
avgm = onemat(:,1)*avg;
xn=x-avgm;
xmult=xn*xn';
dist=sqrt(diag(xmult));

% for i=1:size(x,1)
%     dist(i,:)=  norm(x(i,:)-avg);  %Distances of each point in the dataset from the mean
% end

maxdist = max(dist);            %Finding the maximum distance from all the distances computed 
scalestep=maxdist/(size(x,1)*0.66); %scalestep for the histogram
scale = 1:scalestep:maxdist;         %X axis scale for the histogram

for i=1:size(x,1)
    mainindex(i,1) = i;         %Dummy matrix that has values from 1 to size of dataset in the format 1,2,3,.. for 
end                             %indexing purposes

cluster(1:size(x,1),1) = 0;                    %Initialization
centroid(1:size(x,1),1) = 0;
value = 0;
no = 1;                   

[cluster, no, centroid] = recurauto1(x,dist,scale,mainindex,cluster,no,centroid,value,mnp,percentage); %Calling recursive function

totalnoofclusters =  no - 1

for i = 1:totalnoofclusters         %Check to see how many points are in the cluster, before mapping
pts_in_cluster_before_reallocation = size(cluster(find(cluster == i),:),1); %unclustered data points
end

pts_not_clustered = size(cluster(find(cluster == 0),:),1)  %Points left unclustered

if size(centroid(find(cluster == 0)),1) ~= 0      %Distances of all the unclustered datapoints from the 
for i=1:totalnoofclusters                         %centroids are found and the datapoints are mapped to the
    temp = 0;                                     %closest cluster 
    temp = centroid(find(cluster==i));
%    cl = 0;
    y = x(find(cluster == 0),:);
    for j=1:size(y,1)
        distance(i,j) = norm(y(j,:) - x(temp(1),:));
    end
end

[mini, pos] = min(distance);

position = find(cluster == 0);

if size(distance,1) == 1
    pos(1:size(position,1)) = 1;
end

for i = 1:size(position,1)
temp = centroid(find(cluster == pos(1,i)),:);
centroid(position(i,1),:) = temp(1);       %Computing centroids for unclustered data points
end
cluster(find(cluster == 0),:) = pos';      %Remapping of unclustered data points
end

total_no_of_pts_in_all_clusters = 0;          %Computes the total datapoints in all clusters
for i = 1:totalnoofclusters
    total_no_of_pts_in_all_clusters = total_no_of_pts_in_all_clusters + size(cluster(find(cluster == i),:),1);
end

total_no_of_pts_in_all_clusters
%x(find(cluster == 1),:)
%temp = centroid(find(cluster==1));x(temp(1),:)


%figure

color = ['ok','ob','og','or','oc','om','oy'
         '+k','+b','+g','+r','+c','+m','+y';
         'xk','xb','xg','xr','xc','xm','xy'];  %Matrix of colors used for outputting purposes


%cl = 0;
%cl=datas(find(cluster == 1),:); size(cl,1)             %Plotting 3D output for both clusters and centroids 
 %   plot3(cl(:,1),cl(:,2),cl(:,3),'*'); hold
    
    j = 1; k = 1;
    for i=2:totalnoofclusters
        if i == 9 
            j = 2; k = 1;
        elseif i == 16
            j = 3; k = 1;
        end
%        cl=datas(find(cluster == i),:); size(cl,1)
       % plot3(cl(:,1),cl(:,2),cl(:,3),[color(j,k) color(j,k+1)]);
        k=k+2;
    end
    
    for i=1:totalnoofclusters
        temp = 0;
        temp = centroid(find(cluster==i));
%        cl = 0;
%        cl=datas(temp(1),:);
       % plot3(cl(:,1),cl(:,2),cl(:,3),'ob');
    end
%    rotate3d
