% Jeremy W. Sheaffer 11/4/2002
%
% extract most relevant keyframes from a video using clustering
% Uses Dr. Longin Jan Latecki's new clustering algorithm firstauto1
%
%Example: latecki_global('mrbeantu.bft',100,0.1)
function keyframes = latecki_global(feature_file, num_frames, p)

close all;

a = load(feature_file);

s = std(a);
q = find(s ~= 0);
data = a(:, q);

for i = 4 : 3 : length(q)
  datas(:, i) = data(:, i) / 1000;
  datas(:, i - 2) = data(:, i - 2) .* datas(:, i) / 1000;
  datas(:, i - 1) = data(:, i - 1) .* datas(:, i) / 1000;   
end

[m, s, datas] = normalize(datas, [], []);

[cluster_table] = firstauto1(datas, num_frames, p);

frame_count = zeros(max(cluster_table));

for i = 1 : size(cluster_table, 1)
  frame_count(cluster_table(i)) = frame_count(cluster_table(i)) + 1;
  cluster_frame_table(frame_count(cluster_table(i)), cluster_table(i)) ...
      = i;
end

for i = 1 : max(cluster_table)
  kf = cluster_frame_table(find(cluster_frame_table(:, i) ~= 0), i);
  for k = 1 : size(datas, 2)
    center(k) = 0;
    for j = 1 : length(kf)
      center(k) = center(k) + datas(kf(j), k);
    end
    center(k) = center(k) / length(kf);
  end
  closest_dist = inf;
  for k = 1 : length(kf)
    dist = 0;
    for j = 1 : size(datas, 2)
      dist = dist + (datas(kf(k), j) - center(j)) ^ 2;
    end
    dist = sqrt(dist);
    if dist < closest_dist
      closest_dist = dist;
      f(i) = kf(k);
    end
  end
end

keyframes = sort(f);

return
